package geniusweb.domaineditor.model.profile;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import geniusweb.domaineditor.model.DiscreteValueSetModel;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.profile.utilityspace.DiscreteValueSetUtilities;
import geniusweb.profile.utilityspace.ValueSetUtilities;
import tudelft.utilities.listener.DefaultListenable;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.mvc.model.ListModel;
import tudelft.utilities.mvc.model.MapModel;
import tudelft.utilities.mvc.model.NumberModel;
import tudelft.utilities.mvc.model.RestrictedNumberModel;
import tudelft.utilities.mvc.model.StringModel;
import tudelft.utilities.mvc.model.events.Event;
import tudelft.utilities.mvc.panels.PopupReporter;

/**
 * 
 * Contains info for a {@link DiscreteValueSetUtilities} as a map. The keys of
 * the map are the issuevalues in the {@link DiscreteValueSetModel}, the values
 * are the utilities stored in this model.
 *
 */
public class DiscreteValueSetUtilitiesModel extends DefaultListenable<Event>
		implements MapModel<StringModel, RestrictedNumberModel>,
		ValueSetUtilitiesModel {
	private final ListModel<StringModel> values;
	private final Map<StringModel, RestrictedNumberModel> utilities = new HashMap<>();
	private final Reporter log;
	private static final BigDecimal DEFAULT = new BigDecimal("0.5");
	// this reporter in theory is never used.
	// it's connected to values that never change
	private final static Reporter dummylog = new PopupReporter(null);
	private final static NumberModel LOW = new NumberModel(BigDecimal.ZERO,
			dummylog);
	private final static NumberModel HIGH = new NumberModel(BigDecimal.ONE,
			dummylog);

	public DiscreteValueSetUtilitiesModel(DiscreteValueSetModel values,
			Reporter log) {
		this.values = values.getList();
		this.log = log;
	}

	@Override
	public RestrictedNumberModel getValue(StringModel key) {
		if (!utilities.containsKey(key)) {
			utilities.put(key,
					new RestrictedNumberModel(DEFAULT, LOW, HIGH, log));
		}
		return utilities.get(key);
	}

	@Override
	public void put(StringModel key, RestrictedNumberModel val) {
		throw new UnsupportedOperationException();
	}

	@Override
	public void remove(StringModel key) {
		throw new UnsupportedOperationException();
	}

	@Override
	public ListModel<StringModel> getKeys() {
		return values;
	}

	@Override
	public int getMinimumSize() {
		return 1;
	}

	@Override
	public String getColumnName(int n) {
		return n == 0 ? "value" : "utility";
	}

	@Override
	public DiscreteValueSetUtilities getCurrentValue()
			throws IllegalStateException {
		Map<DiscreteValue, BigDecimal> map = new HashMap<>();
		for (int n = 0; n < values.getSize(); n++) {
			StringModel val = values.get(n);
			map.put(new DiscreteValue(val.getValue()),
					getValue(val).getValue());
		}

		return new DiscreteValueSetUtilities(map);
	}

	@Override
	public void setCurrentValue(ValueSetUtilities obj)
			throws IllegalArgumentException {
		DiscreteValueSetUtilities vutils = (DiscreteValueSetUtilities) obj;
		for (int n = 0; n < values.getSize(); n++) {
			StringModel val = values.get(n);
			// FIXME this silently uses util 0 if value not in bid. Maybe warn?
			BigDecimal util = vutils
					.getUtility(new DiscreteValue(val.getValue()));
			getValue(val).setValue(util);
		}

	}

	@Override
	public RestrictedNumberModel create(StringModel key) {
		throw new UnsupportedOperationException();
	}

}
